# OBJ - timestamped JSON objects
#
# obj/clock.py
#
# Copyright 2017,2018 B.H.J Thate
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice don't have to be included.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 13-07-2018 As the creator of this file, I disclaim all rights on  this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" timer, repeater. timed daemon. """

__version__ = 1

## imports

import obj.base
import threading
import time

## classes

class Timers(obj.base.Dotted):

    def __init__(self, *args, **kwargs):
        self._name = "Timers"
        self._stopped = False
        self.cfg = obj.base.Dotted()
        self.timers = obj.base.Dotted()
        self.ready()

    def loop(self):
        while not self._stopped:
            time.sleep(1.0)
            remove = []
            for t, event in self.timers.items():
                if time.time() > t:
                    cfg.latest = time.time()
                    cfg.save()
                    event.raw(event.txt)
                    remove.append(t)
            for r in remove:
                del self.timers[r]
        
    def start(self):
        store = obj.base.Store()
        for e in store.last("obj.clock.Timers"):
            if "done" in e and e.done:
                continue
            if "time" not in e:
                continue
            if time.time() < int(e.time):
                self.timers[e.time] = e
        return obj.task.launch(self.loop)
        
    def stop(self):
        self._stopped = True

class Timer(obj.base.Dotted):

    def __init__(self, sleep, func, *args, **kwargs):
        super().__init__()
        self._func = func
        self._name = kwargs.get("name", obj.utils.name(func))
        self.sleep = sleep
        self.args = args
        self.kwargs = kwargs
        try:
            self._event = self.args[0]
        except:
            self._event = obj.event.Event()
        self._time = obj.base.Dotted()

    def start(self, *args, **kwargs):
        timer = threading.Timer(self.sleep, self.run, self.args, kwargs or self.kwargs)
        timer.setName(self._name)
        timer.sleep = self.sleep
        timer._event = self._event
        timer._time = self._time
        timer._time.start = time.time()
        timer._time.latest = time.time()
        timer._func = self._func
        timer.start()
        return timer

    def run(self, *args, **kwargs):
        self._time.latest = time.time()
        obj.task.launch(self._func, *args, **kwarg)

    def exit(self):
        self.cancel()

class Repeater(Timer):

    def run(self, *args, **kwargs):
        self._func(*args, **kwargs)
        obj.task.launch(self.start)
