# OBJ - timestamped JSON objects
#
# obj/cmds.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    basic commands.
    
"""

__version__ = 1

## imports

import obj.base
import obj.fleet
import obj.shell
import obj.users
import obj.utils

import logging
import os
import time

## decorators

def checkargs(func, *args, **kargs):

    store = obj.base.Store()

    def check(*args, **kwargs):
        if args:
            event = args[0]
            if not event.args:
                res = sorted([x.split(".")[-1].lower() for x in os.listdir(obj.base.cfg.workdir)])
                event.reply("%s <%s> [match]" % (event.get_cmd(), ",".join(res)))
                return
            tt = type = event.args[0]
            for t in sorted([x for x in os.listdir(obj.base.cfg.workdir) if type in x.split(".")[-1].lower()]):
                if type in t.lower():
                    type = t
            if len(event.args) == 1:
                o = store.last(type)
                if o:
                    res = sorted(set([x for x in o if not x.startswith("_")]))
                    event.reply("%s %s [%s]=value" % (event.get_cmd(), event.args[0], ",".join(res)))
                    return
        return func(*args, **kwargs)

    return check

## classes

class Log(obj.base.Dotted):

    pass

class Todo(obj.base.Dotted):

    pass

## commands

def announce(event):
    """ annouce txt to listeners. """
    fleet = obj.fleet.Fleet()
    fleet.announce(event.rest)

def cmds(event):
    """ show commands. """
    fleet = obj.fleet.Fleet()
    bot = fleet.get_bot(event.orig)
    event.reply(",".join(sorted(bot.names)))

def cfg(event):
    """ cfg <type>. """
    if not event.args:
        event.reply("cfg <name>")
        return
    name = "obj.%s.Cfg" % event.args[0]
    store = obj.base.Store()
    try:
        c = store.last(name)
        event.reply(c)
    except FileNotFoundError:
        event.reply("%s config not found" % event.args[0])

def ed(event):
    """ edit objects. """
    store = obj.base.Store()
    try:
        o = store.last(event.args[0])
    except IndexError:
        event.reply("ed <type>")
        return
    except IsADirectoryError as ex:
        event.reply("%s %s" % (event.txt, ex.strerror.lower()))
        return
    if not o:
        event.reply("no match found for %s" % " ".join(event.args))
        return
    if len(event.args) == 1:
        event.reply(o)
        return 
    key = event.args[1]
    if key not in o:
        event.reply("no %s attribute available." % key)
        return
    if len(event.args) == 2:
        event.reply('%s=%s' % (key, o[key]))
        return
    if len(event.args) == 3:
        val = event.args[2]
    elif len(event.args) >= 3:
        val = event.args[2:]
    try:
        val = int(val)
    except:
        try:
            val = float(val)
        except:
            pass
    t = type(o.get(key))
    tt = type(val)
    if t != tt:
        event.reply("%s != %s" % (tt, t))
        return
    if t == str:
        o.set(key, val)
    elif t == list:
        o.set(key, val)
    elif val in ["True", "true"]:
        o.set(key, True)
    elif val in ["False", "false"]:
        o.set(key,  False)
    else:
        try:
            val = '"%s"' % val
            o[key] = json.loads(val)
        except json.decoder.JSONDecodeError:
            event.reply("%s is not JSON" % val)
        except (SyntaxError, ValueError, json.decode.JSONDecodeError) as ex:
            event.reply("%s: %s" % (str(ex), val))
        return
    o.save()
    event.reply('ok %s="%s"' % (key, o.get(key)))

def fleet(event):
    """ show bots on fleet. """
    bots = obj.fleet.Fleet()
    try:
        nr = int(event.args[0])
    except (IndexError, ValueError) as ex:
        event.reply(str([type(b) for b in bots.bots]))
        return
    try:
        event.reply(bots.bots[nr])
    except IndexError:
        pass

@checkargs
def find(event):
    """ find <prefix> [match] - find objects with prefix and matching fields. """
    nr = 0
    store = obj.base.Store()
    for o in store.find(event.want, event.type):
        if "d" in event.options and "_deleted" in o and not o._deleted:
            continue
        if "t" in event.options:
            days = obj.utils.elapsed(time.time() - obj.utils.fn_time(o._container.get("path", 0)))
        else:
            days = 0
        if days:
            event.display(o, txt=str(nr), post="- %s" % days)
        else:
            event.display(o, txt=str(nr))
        nr += 1
        
def kill(event):
    """ kill a task. """
    if not event.args:
        event.reply("stop <match>")
        return
    n = event.args[0]
    obj.task.kill(n)
    event.reply("ok %s" % n)

def load(event):
    """ load modules. """
    fleet = obj.fleet.Fleet()
    bot = fleet.get_bot(event.orig)
    if not event.args:
        event.reply("load <modname>")
        return
    name = event.args[0]
    try:
        mod = bot.load_mod(name)
        if "init" in dir(mod):
            mod.init()
        obj.shell.set_completer(bot.names)
        event.reply("%s loaded" % name)
    except ModuleNotFoundError:
        event.reply("%s doesn't exist." % name)

def log(event):
    """ log some text. """
    l = Log()
    l.txt = event.rest
    l.save()
    event.ok("1")

def ls(event):
    """ show list of workdir subdirectories. """
    res = sorted(set([x.split(".")[-1].lower() for x in os.listdir(obj.base.cfg.workdir)]))
    if not event.args:
        if res:
            event.reply(",".join(res))
    else:
        m = event.args[0].lower()
        event.reply(",".join(set([x.split(".")[-1].lower() for x in os.listdir(obj.base.cfg.workdir) if m in x.lower()])))

def meet(event):
    """ add a new user. """
    try:
        origin, *perms = event.args
    except ValueError:
        event.reply("meet <origin> <perms>")
        return
    users = obj.users.Users()
    origin = users.userhosts.get(origin, origin)
    if users.meet(origin, perms):
        event.reply("%s added." % origin)

def perm(event):
    """ set permission of a user. """
    try:
        origin, perms = event.args
    except ValueError:
        event.reply("perm <origin> <perm>")
        return
    users = obj.users.Users()
    origin = users.userhosts.get(origin, origin)
    u = users.perm(origin, perms)
    if u:
        event.reply("ok %s" % origin)

def ps(event):
    """ show running threads. """
    psformat = "%-8s %-60s"
    result = []
    try:
        getnr = int(event.args[0])
    except (ValueError, IndexError):
        getnr = None
    for thr in sorted(obj.task.running(), key=lambda x: x.getName()):
        o = obj.base.Dotted()
        o.update(vars(thr))
        if getattr(o, "sleep", None):
            up = o.sleep - int(time.time() - o._time.latest)
        else:
            up = int(time.time() - obj.base.starttime)
        thrname = thr.getName()
        result.append((up, thrname, o))
    nr = -1
    for up, thrname, o in sorted(result, key=lambda x: x[0]):
        nr += 1
        if getnr != None and getnr == nr:
            event.reply(o)
            return
        res = "%s  %s" % (nr, psformat % (obj.utils.elapsed(up), thrname[:30]))
        event.reply(res.rstrip())

@checkargs
def rm(event):
    """ rm <prefix> [match] - set deleted flag. """
    nr = 0
    starttime = time.time()
    store = obj.base.Store()
    for o in store.find(event.want, event.cls, strict=True):
        o._deleted = True
        o.save()
        nr += 1
    event.reply("ok %s %s" % (nr, obj.utils.elapsed(time.time()-starttime)))

def rmperm(event):
    """ remove permission of a user. """
    try:
        origin, perm = event.args
    except:
        event.reply("rmperm <origin> <perm>")
        return
    users = obj.users.Users()
    origin = users.userhosts.get(origin, origin)
    result = users.delete(origin, perm)
    if not result:
        return
    event.reply("ok %s %s" % (origin, perm))

def save(event):
    """ save all bots on the fleet. """
    nr = 0
    fleet = obj.fleet.Fleet()
    for bot in fleet.bots:
        try:
            bot.cfg.save()
            nr += 1
        except AttributeError:
            pass
    event.reply("%s bots saved." % nr)

def test(event):
    """ echo test response. """
    event.reply("yoo %s" % event.origin)

def todo(event):
    """ log a todo item. """
    t = Todo()
    t.txt = event.rest
    t.save()
    event.ok("1")

@checkargs
def undel(event):
    """ undelete JSON objects.  """ 
    nr = 0
    starttime = time.time()
    store = obj.base.Store()
    for o in store.find(event.want, event.cls, skip=False, strict=True):
        o._deleted = False
        o.save()
        nr += 1
    event.reply("ok %s %s" % (nr, obj.utils.elapsed(time.time()-starttime)))

def unload(event):
    """ unload modules. """
    fleet = obj.fleet.Fleet()
    bot = fleet.get_bot(event.orig)
    if not event.args:
        event.reply("unload <modname>")
        return
    name = event.args[0]
    if name == "obj.cmds":
        event.reply("can't unload obj.cmds")
        return
    try:
        bot.unload(name)
        obj.shell.set_completer(bot.names.keys())
    except KeyError:
        event.reply("%s is not unloadable." % name)
        return
    event.reply("%s unloaded" % name)

def uptime(event):
    """ show uptime. """
    event.reply(obj.utils.elapsed(time.time() - obj.base.starttime))

def user(event):
    """ show user. """
    if not event.args:
        event.reply("user <origin>")
        return
    users = obj.users.Users()
    origin = event.args[0]
    origin = users.userhosts.get(origin, origin)
    for u in users.get_users(origin):
        event.reply(u)

def version(event):
    """ show version. """
    event.reply("OBJ #%s - %s" % (obj.base.__version__, obj.base.__txt__))
