# OBJ - timestamped JSON objects
#
# obj/users.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-07-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" register an user and give them USER or OPER permission. """

__version__ = 1

## imports

import obj.base
import logging
import os

# classes

class User(obj.base.Dotted):

    pass

class Users(obj.base.Store):

    userhosts = obj.base.Dotted()

    def allowed(self, origin, perm):
        perm = perm.upper()
        for user in self.get_users():
            if origin == user.user and perm in user.perms:
                logging.warn("allow %s %s" % (origin, perm))
                return True
        logging.error("denied %s %s" % (origin, perm))
        return False

    def delete(self, origin, perm):
        for user in self.get_users(origin):
            try:
                user.perms.remove(perm)
                user.save()
                return True
            except ValueError:
                pass
        return user

    def get_users(self, origin=""):
        m = obj.base.Dotted()
        m.user = origin
        return self.find(m, "obj.users.User")

    def meet(self, origin, perms=[]):
        user = User()
        user.user = origin
        user.perms = ["USER", ]
        user.perms.extend(perms)
        user.save()
        return user

    def oper(self, origin):
        user = User()
        user.user = origin
        user.perms = ["OPER", "USER"]
        return user

    def perm(self, origin, permission):
        for user in self.get_users(origin):
            if permission.upper() not in user.perms:
                user.perms.append(permission.upper())
                user.save()
                return user
